@@ -3,7 +3,7 @@ import math
 import os
 from collections import defaultdict
 from pathlib import Path
-from typing import Tuple, Dict, Any, List
+from typing import Tuple, Dict, Any, List, Type
 
 import aiofiles
 from gcp_pilot.datastore import Document, DoesNotExist
@@ -20,8 +20,16 @@ ResponseType = Tuple[PayloadType, int]
 STAGE_DIR = os.environ.get('SANIC_FILE_DIR', default=Path(__file__).parent)
 
 
+def get_model_or_404(model_klass: Type[Document], pk: str, query_filters: Dict = None) -> Document:
+    query_filters = query_filters or {}
+    try:
+        return model_klass.documents.get(id=pk, **query_filters)
+    except DoesNotExist as e:
+        raise exceptions.NotFoundError() from e
+
+
 class ViewBase(HTTPMethodView):
-    model: Document
+    model: Type[Document]
 
     @classmethod
     async def write_file(cls, file: File, filepath: Path):
@@ -53,6 +61,17 @@ class ViewBase(HTTPMethodView):
 
         return errors
 
+    def _parse_pk(self, pk: str):
+        pk_field_name = self.model.Meta.pk_field
+        pk_field_type = self.model.Meta.fields[self.model.Meta.pk_field]
+        try:
+            return pk_field_type(pk)
+        except ValueError as e:
+            raise exceptions.ValidationError(f"{pk_field_name} field must be {pk_field_type.__name__}") from e
+
+    def get_model(self, pk: str) -> Document:
+        return get_model_or_404(model_klass=self.model, pk=self._parse_pk(pk))
+
 
 class ListView(ViewBase, abc.ABC):
     search_field: str = None
@@ -142,21 +161,6 @@ class ListView(ViewBase, abc.ABC):
 
 
 class DetailView(ViewBase, abc.ABC):
-    def _parse_pk(self, pk: str):
-        pk_field_name = self.model.Meta.pk_field
-        pk_field_type = self.model.Meta.fields[self.model.Meta.pk_field]
-        try:
-            return pk_field_type(pk)
-        except ValueError as e:
-            raise exceptions.ValidationError(f"{pk_field_name} field must be {pk_field_type.__name__}") from e
-
-    def get_model(self, pk: str, query_filters: Dict = None) -> Document:
-        query_filters = query_filters or {}
-        try:
-            return self.model.documents.get(id=self._parse_pk(pk=pk), **query_filters)
-        except DoesNotExist as e:
-            raise exceptions.NotFoundError() from e
-
     async def get(self, request: Request, pk: str) -> HTTPResponse:
         current_obj = self.get_model(pk=pk)
 
@@ -240,70 +244,120 @@ class DetailView(ViewBase, abc.ABC):
         self.model.documents.delete(pk=pk)
 
 
-class ActionView(ViewBase):
-    def get_model(self, pk: str) -> Document:
-        return self.model.documents.get(id=pk)
+class NestViewBase(ViewBase):
+    nest_model: Type[Document]
 
-    async def get(self, request: Request, pk: str) -> HTTPResponse:
-        try:
-            obj = self.get_model(pk=pk)
-        except DoesNotExist as e:
-            raise exceptions.NotFoundError() from e
+    def get_nest_model(self, pk: str):
+        return get_model_or_404(model_klass=self.nest_model, pk=pk)
+
+
+class NestedListView(NestViewBase):
+    async def get(self, request: Request, nest_pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
 
         try:
-            data, status = await self.perform_get(request=request, obj=obj)
+            data, status = await self.perform_get(request=request, nest_obj=nest_obj)
         except ValidationError as e:
             raise exceptions.ValidationError(message=str(e))
 
         return json(data, status)
 
     @abc.abstractmethod
-    async def perform_get(self, request: Request, obj: Document) -> ResponseType:
+    async def perform_get(self, request: Request, nest_obj: Document) -> ResponseType:
         raise NotImplementedError()
 
-    async def post(self, request: Request, pk: str) -> HTTPResponse:
-        try:
-            obj = self.get_model(pk=pk)
-        except DoesNotExist as e:
-            raise exceptions.NotFoundError() from e
+    async def post(self, request: Request, nest_pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
 
         try:
-            data, status = await self.perform_post(request=request, obj=obj)
+            data, status = await self.perform_post(request=request, nest_obj=nest_obj)
         except ValidationError as e:
             raise exceptions.ValidationError(message=str(e))
 
         return json(data, status)
 
     @abc.abstractmethod
-    async def perform_post(self, request: Request, obj: Document) -> ResponseType:
+    async def perform_post(self, request: Request, nest_obj: Document) -> ResponseType:
         raise NotImplementedError()
 
-    async def put(self, request: Request, pk: str) -> HTTPResponse:
-        try:
-            obj = self.get_model(pk=pk)
-        except DoesNotExist as e:
-            raise exceptions.NotFoundError() from e
+    async def put(self, request: Request, nest_pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
 
         try:
-            data, status = await self.perform_put(request=request, obj=obj)
+            data, status = await self.perform_put(request=request, nest_obj=nest_obj)
         except ValidationError as e:
             raise exceptions.ValidationError(message=str(e))
 
         return json(data, status)
 
     @abc.abstractmethod
-    async def perform_put(self, request: Request, obj: Document) -> ResponseType:
+    async def perform_put(self, request: Request, nest_obj: Document) -> ResponseType:
         raise NotImplementedError()
 
-    async def delete(self, request: Request, pk: str) -> HTTPResponse:
-        try:
-            obj = self.get_model(pk=pk)
-        except DoesNotExist as e:
-            raise exceptions.NotFoundError() from e
+    async def delete(self, request: Request, nest_pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
 
-        data, status = await self.perform_delete(request=request, obj=obj)
+        data, status = await self.perform_delete(request=request, nest_obj=nest_obj)
         return json(data, status)
 
     @abc.abstractmethod
-    async def perform_delete(self, request: Request, obj: Document) -> ResponseType:
+    async def perform_delete(self, request: Request, nest_obj: Document) -> ResponseType:
+        raise NotImplementedError()
+
+
+class NestedDetailView(NestViewBase):
+    async def get(self, request: Request, nest_pk: str, pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
+        obj = self.get_model(pk=pk)
+
+        try:
+            data, status = await self.perform_get(request=request, nest_obj=nest_obj, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
+        return json(data, status)
+
+    @abc.abstractmethod
+    async def perform_get(self, request: Request, nest_obj: Document, obj: Document) -> ResponseType:
+        raise NotImplementedError()
+
+    async def post(self, request: Request, nest_pk: str, pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
+        obj = self.get_model(pk=pk)
+
+        try:
+            data, status = await self.perform_post(request=request, nest_obj=nest_obj, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
+        return json(data, status)
+
+    @abc.abstractmethod
+    async def perform_post(self, request: Request, nest_obj: Document, obj: Document) -> ResponseType:
+        raise NotImplementedError()
+
+    async def put(self, request: Request, nest_pk: str, pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
+        obj = self.get_model(pk=pk)
+
+        try:
+            data, status = await self.perform_put(request=request, nest_obj=nest_obj, obj=obj)
+        except ValidationError as e:
+            raise exceptions.ValidationError(message=str(e))
+
+        return json(data, status)
+
+    @abc.abstractmethod
+    async def perform_put(self, request: Request, nest_obj: Document, obj: Document) -> ResponseType:
+        raise NotImplementedError()
+
+    async def delete(self, request: Request, nest_pk: str, pk: str) -> HTTPResponse:
+        nest_obj = self.get_nest_model(pk=nest_pk)
+        obj = self.get_model(pk=pk)
+
+        data, status = await self.perform_delete(request=request, nest_obj=nest_obj, obj=obj)
+        return json(data, status)
+
+    @abc.abstractmethod
+    async def perform_delete(self, request: Request, nest_obj: Document, obj: Document) -> ResponseType:
         raise NotImplementedError()
